function [c,l,v,h,x, c1N,muN,phiXN,BTN,lambdaN,dV_dh1N,valAbil,hT1]=...
    hhiter(Tr,c1,w,r,tl,xEarn,xDeduct,sx,tcP1,mu,phiX,hIn,a1,abil,...
    B1,tBequ,tbP1,hAvg,VPrime,bequ,BT,vh1Child,lambda,T, dbg);
% hhiter: One iteration over the hh problem;
% ---------------------------------------
% TASK:
%  Solve the hh problem, taking shadow prices as given;
%  produces new shadow prices
%  and the time allocation that solves the hh problem, given those
%  shadow prices
%  Does not contain investment in h1

% IN:
%   Tr      = definition as usual
%   c1      = c(1)
%   w,r     = factor prices; 1xT
%   mu      = shadow prices; 1xT; mu*
%   phiX    = shadow prices for h; phi * lambda / R
%   hIn     = guess for h. 1xT
%   a1      = initial wealth (NOT a1*(1+r)!)
%   abil    = ability in G()

%   bequest related variables -> hhprob.m
%   - bequ:    the same as the global; set this to zero if the problem
%              shall be solved under the assumption that the bequest motive is
%              not operative
%   - BT:      bequest for which VPrime is the correct value
%   - VPrime   parent values bequest at (bequ * VPrime / (1+tb))

%   hAvg       average h in the economy at age t; 1xT
%   vh1Child   dV/dh1 of one child
%   lambda     see outputs; NOT for child!

% OUT:
%   (c,l,v,x,h) which solve the hh problem
%   c1: new guesses
%   mu: new guesses
%   phiXN: new guesss
%   hT1:    h(T+1)
%   lambdaN             dV/da1; derivative of value function
%   dV_dh1              dV/dh1 = phi0


% REM:
% - phiX in this program corresponds to phi* = value of hc evaluated at end of age t

% - Note the distinction between hIn and h. h is the new guess. Almost everywhere hIn should
%   be used. This stabilizes the algorithm.

% TEST:   t_hhiter
% AUTHOR: Lutz Hendricks, 1995-97
% ---------------------------------------

% *** Globals ***

global bb sig theta rho tb
global B psi eta ddh ksi zeta gg
global hcAge hcInherit hAbility popGrowth tb hSpill
global ALTRUISTIC NONALTRUISTIC NOBEQUEST BEQUTYPE hcAltruism

updateBT = 01;     % Update BT or take as given?


% ***** INPUT CHECK *****

if dbg > 5
   if any( mu<0 )
       abort('Negative mu')
   end
   if any( iscomplx([mu,phiX]) )
       abort('Complex mu or phi inputs')
   end
   if iscomplx(VPrime) | VPrime<0
      warnmsg('Invalid VPrime')
      keyboard
      abort('Aborted')
   end
end


% ******* CONSTANTS ********

   RoundoffTol = 1e-6;
   psiEta2 = 1/(1 - psi - eta);
   Bgf  = B .* abil.^gg .* hAvg.^zeta .* hIn.^ksi;


   exp1 = 1 / (rho*(1-sig)-sig);
   exp2 = rho*(sig-1);
   r1  = 1 + r;
   Ri  = cumprod(r1);
   % ** Private cost of a unit of x
   cx = x_cost(tl, sx, xEarn, xDeduct);
   if dbg > 5
      if any(cx <= 0)
          cx
          abort('Negative cx')
      elseif any(cx <= 0.05)
          warnmsg('HHIter: cx <= 0.05')
      end
   end
   wstar = w .* hIn + mu;


% ************* SOLUTION OF HH PROBLEM  ****************

   v = zeros(1,T);
   phiPosIdx = find( phiX > 0 );
   tmp = wstar(phiPosIdx) ./ psi ./ Bgf(phiPosIdx) ./ phiX(phiPosIdx);
   v(phiPosIdx) = (psi./eta .* cx(phiPosIdx)./wstar(phiPosIdx)).^(eta/(psi+eta-1)) ...
      .* tmp .^ (1/(psi+eta-1));

   tmp = find( v > 1 );
   if ~isempty( tmp )
      v(tmp) = v(tmp) .^ 0.5;
   end
   tmp = find( v < RoundoffTol );
   if ~isempty( tmp )
      v(tmp) = 0;
   end

   x = zeros(1,T);
   x(phiPosIdx) = v(phiPosIdx) .* eta./psi .* wstar(phiPosIdx) ./ cx(phiPosIdx);

   % Depends on functional form: Gf
   G = zeros(1,T);
   G(phiPosIdx) = Bgf(phiPosIdx) .* v(phiPosIdx).^psi .* x(phiPosIdx).^eta;

   %h = zeros(1,T);
   %h(1) = hIn(1);
   %for i = 1 : T-1
   %   h(i+1) = (1-ddh) * h(i) + G(i);
   %end

   % The same code without the loop
   h = [hIn(1), hIn(1) .* (1-ddh).^(1:T-1) + (1-ddh).^(1:T-1) .* cumsum( 1 ./ (1-ddh).^(1:T-1) .* G(1:T-1) )];
   hT1 = h(T) * (1-ddh) + G(T);


   tmp = (tcP1(2:T)./tcP1(1:T-1)) .^ (1+exp2) ./ bb ./ r1(2:T) ...
         .* (wstar(1:T-1) ./ wstar(2:T)) .^ exp2;
   c = [c1, c1 .* cumprod( tmp .^ exp1 )];
   l = c .* rho .* tcP1 ./ wstar;

   if dbg > 5
      if any( iscomplx([l,v]) )
         abort('Complex l,v')
      end
   end

   wtime = 1 - l - v;
   wtimePos = max( wtime, 0 );


   % *******************  Update guesses for phi  *****************
   % phiX corresponds to (phi / lambda * R)

   phiXN = zeros(1,T);
   RHS = w .* wtimePos;

   if hcAltruism == 1
      %vh1Factor = bequ * (bb * popGrowth)^(tb-1) * hcInherit * vh1Child / gr^(sig*(tb-1));
      vh1Factor = vh1Child;
      if hSpill.type == 1
          % Value the parent assigns to h1 of child
          % transformed so that it can be added to phi
          if hcAge == T+1
             phiXN(T) = vh1Factor * Ri(T) / lambda;
          else
             vh1Factor = vh1Factor * Ri(hcAge) / lambda;
             RHS(hcAge) = RHS(hcAge) + vh1Factor;
          end
      else
         % ** Spillover over range of dates
         % Weights by age (betw 1 and 0)
         % The term ".*Ri ./ lambda" divides by lambdaBar(i); b/c phiX is transformed here
         weightV = zeros(1,T);
         weightV(hSpill.age1:hSpill.age2) = (hSpill.age2 - (hSpill.age1:hSpill.age2)) ./ ...
            (hSpill.age2 - hSpill.age1);
         RHS = RHS + vh1Factor .* Ri ./ lambda .* weightV;
      end
   end


   % Using phiX(t) to update is slower (and has no effect outside of hhprob)
   Gh = ksi .* G ./ h;

   for t = T : (-1) : 2
      phiXN(t-1) = ( phiXN(t) * (1-ddh + Gh(t)) + RHS(t) ) / r1(t);
   end


   % ** Value of h1 at date 0
   phiX0N = ( phiXN(1) * (1-ddh + Gh(1)) + RHS(1) ) / r1(1);
   if hAbility == 1
       % ** Value of ability; discounted to date 0; in terms of goods
       Gz = gg .* G ./ abil;
       valAbil = phiXN(1:T-1) * Gz(1:T-1)'  ./ r1(1);
   else
       valAbil = 0;
   end



% *********************  Bequest  *****************************

   if BEQUTYPE == NOBEQUEST
      BTN = 0;
   else
      if updateBT == 0
         BTN = BT;
      elseif BEQUTYPE == NONALTRUISTIC
         % ** Non-altruistic bequest => FOC gives BTN
         [ tmp, ucT ] = ufct( c(T), l(T) );
         BTN = ( bequ*tcP1(T)/tbP1 / ucT ) ^ (1/sig);
      elseif BEQUTYPE == ALTRUISTIC
         % Parent values bequest at bequ*VPrime/(1+tb)
         [ tmp, ucT ] = ufct( c(T), l(T) );
         MuRatio = bb^T * ucT / tcP1(T) / (bequ * VPrime / tbP1);
         if BT > RoundoffTol
             BTN = BT / MuRatio;
         else
             BTN = c(T);
         end
      else
         abort('Unknown BEQUTYPE')
      end
   end


   % ******************* Update c1 from b.c. **********************
   % ** Nonaltruistic bequest:
   % **   BTN always > 0. BTN can be computed from FOC. => use standard
   % **   method.
   % ** Altruistic bequest:
   % **   No FOC for BTN. BTN must be computed as a residual from the b.c.
   % **   Guess for BTN may be < 0. Then adjust c1 until BTN>=0

   % bequ endogenous
   if bequ <= 0  |  BEQUTYPE == NONALTRUISTIC  |  1
      % Using h instead of hIn is worse
      [ PvExpend, PvIncome ] = hhbc( c,wtimePos,x,hIn, r,w,tl,xDeduct,xEarn,...
         sx, tcP1, Tr, a1*r1(1), B1,tBequ,BTN*tbP1, dbg );
      K = max( PvIncome/PvExpend, 0.05 );
      c1N = c1 * K^0.8;
   elseif  BEQUTYPE == ALTRUISTIC
      % ** Ignore b.c.  Reduce or raise c1 depending on ratio of MUs
      [tmp, ucT] = ufct(c(T),l(T));
      MuRatio = (bb^T * ucT / tcP1(T)) / (bequ*VPrime/tbP1);
      c1N = c1 * MuRatio^0.75;
   else
      abort('Invalid BEQUTYPE when updating c1')
   end


    % ************  UPDATE GUESSES FOR mu*  ************************
    %   if time constraint not violated: mu*=0
    %   else: set mu* s.t. l=1-v and v=1-l. If only one case is used, the
    %   algorithm can get stuck in a corner where the time constraint is
    %   violated, but mu* is not increased.

     %  Construct a factor, K, which is > 1 if the time constraint is
     %  violated and <= 1 otherwise. Use it to update w*. Conclude mu*
     %  from w*.

     % Taking K to an exponent > 1 makes the hh problem less stable. It
     % can lead to oscillations.
     K = (l+v);
     muN = (wstar.*K - w.*hIn);

     % *** Limit change of mu ***
     % **  And make sure that mu gets away from zero when l > 1
     for t = 1 : T
         mut = mu(t);
         muNt = muN(t);
         if mut > RoundoffTol  &  muNt > RoundoffTol
            K = muNt / mut;
            if  K > 20  |  K < 0.1
               muNt = K^0.5 * mut;
            end
         elseif  muNt > 1e10
            muNt = muNt ^ 0.5;
         end
         muN(t) = muNt;
     end


    if dbg > 5
      if  any(iscomplx(muN))
        disp('--- Complex muN -------------------')
        disp(' muN     1-v')
        disp( [muN', 1-v'] )
        abort('---------------------------------------')
      end
    end


% ********  RETURN VALUES  **********

   % Depends on functional form: ufct
   %[ tmp, Uc1 ] = ufct(c1N, l(1));
   Uc1 = c1N ^ (-sig) * l(1) ^ (rho*(1-sig));
   % Untransformed lambda (not lambda/R)
   lambdaN = bb * Uc1 / tcP1(1) * Ri(1);

   % Return derivatives of value function, not multipliers
   % Recall that phiX here is transformed
   dV_dh1N = phiX0N * lambdaN;

% *** end function ***
