function [a1,h1, a,h,x,v] = hhstocks(a1x,h1x,abil,phi0,Rx,hAvg, wx,sxx, dbg)
% Compute optimal h1 and the costs of it
% ---------------------------------------

% IN:
%   a1x     a of age1; discounted to date N+1
%   h1x     h of age1
%   phi0    value of h1 in terms of goods including value of ability shift (if any);
%           discounted to date N
%   Rx      (1+r(t)) for the dates at which investment is feasible, i.e.,
%           for t = 1..N+1
%   hAvg    average h for ages of investment in h1; t=1..N
%   wx      cost of effective labor input, 1xN
%   sxx     subsidy rate on goods, 1xN

% OUT:
%   h1      optimal h1
%   a1      optimal a1 = (a1x - cost of investment in h1) discounted to N+1
%   a       vector of asset holdings ages 1..N
%   h       vector of hc ages 1..N
%   x       vector of goods inputs
%   v       vector of effective time inputs

% DEPENDS ON:
% - Hf

% REM:
% - age N corresponds to date 0
% - investment takes place in periods 1..N
% - values are discounted to date 0 = N
% - H function:
%   - v is EFFECTIVE time purchased!

% TEST:   t_hhstk
% AUTHOR: Lutz Hendricks, 1994-97
% ---------------------------------------

% *** CONSTANTS ***

    global ddhx Bx etax psix ksix gg

    N = length(hAvg);
    ddhx1 = 1 - ddhx;
    sxx1  = 1 - sxx;
    % exponent on hAvg; also subtract psix because v is EFFECTIVE labor input
    zetax = 1 - etax - ksix - gg - psix;



% **************  INPUT CHECK  *****************

 if dbg > 5
     if length(sxx) ~= N
         abort('Invalid length of sxx')
     end
     if length(wx) ~= N
         abort('Invalid length of wx')
     end
     if zetax < 0
         abort('HhStocks: zetax < 0')
     end
 end


 % ** Discount factors from age onwards
 % ** Rfact(age) discounts forward to date N
 Rfact = zeros(1,N);
 Rfact(N) = 1;
 % Eliminate loop
 for age = N-1 : -1 : 1
     Rfact(age) = Rfact(age+1) * Rx(age+1);
 end

 % ** Auxiliary variables
 sxw = sxx1 ./ wx;
 K = Bx .* abil.^gg .* etax .* (psix./etax .* sxw).^psix .* hAvg.^zetax;




% **********************  COMPUTE x  ***********************************

% ** Eliminate the loop **      +++

 x = zeros(1,N);
 v = x;
 h = zeros(1,N+1);
 h(1) = h1x;

 for age = 1 : N
     x(age) = sxx1(age) * ddhx1^(age-N) * Rfact(age) / phi0 / K(age) / h(age)^ksix;
     x(age) = x(age) ^ (1/(etax+psix-1));
     v(age) = x(age) .* sxw(age) .* psix./etax;
     % hf()
     Hf = Bx * abil^gg * x(age)^etax * v(age)^psix * hAvg(age)^zetax * h(age)^ksix;
     h(age+1) = ddhx1*h(age) + Hf;
 end

 h1 = h(N+1);
 h  = h(1:N);


 % *** Asset holdings ***

 a = zeros(1,N);
 cost = x.*sxx1 + v.*wx;
 % ** Start with the PV of a1x at beginning of 1
 a(1) = a1x / prod(Rx(1:N));
 for i = 1 : N-1
     a(i+1) = Rx(i)*a(i) - cost(i);
 end
 % ** Holdings at beginning of N+1
 a1 = Rx(N)*a(N) - cost(N);


% *************************   TEST  *************************************
 if dbg > 5
     %disp(' ')
     %disp('--- HHStocks: Test --------------------------')

     costx = endvalue(cost, Rx(1:N)-1);
     a1_t = a1x - costx;
     if notequal(a1_t, a1, 1e-5)
         disp('** HhStocks:  a1 ~= a1_t **')
         tmp = sprintf('a1: %7.3f   a1_t: %7.3f   Ratio: %7.3f', a1,...
             a1_t, a1/a1_t);
         disp(tmp)
     end

     % ***** Test endvalue of x *****
     evx = cost(1);
     for i = 2 : N
         evx = evx * Rx(i) + cost(i);
     end
     if notequal(evx, costx, 1e-5)
         disp('** HhStocks: Endvalue of x wrong **')
         tmp = sprintf('evx: %7.3f    costx: %7.3f    Ratio: %7.3f',...
             evx, costx, evx/costx);
         disp(tmp);
     end


     % ******  CHECK FOC  ******

     if dbg > 5
         % hf()
         Hf = Bx .* abil.^gg .* x.^etax .* v.^psix .* hAvg.^zetax .* h.^ksix;
         dHdx = etax .* Hf ./ x;
         dHdv = psix .* Hf ./ v;
         dev1 = zeros(1,N);
         dev2 = zeros(1,N);
         for i = 1 : N
             dev1(i) = phi0 .* ddhx1.^(N-i) .* dHdx(i) - ...
                 Rfact(i) .* sxx1(i);
             dev2(i) = phi0 .* ddhx1.^(N-i) .* dHdv(i) - ...
                 Rfact(i) .* wx(i);
         end
         if any(abs(dev1) > 1e-4)
             warnmsg('HHStocks: Deviations in FOC for x')
             dev1
         end
         if any(abs(dev2) > 1e-4)
             warnmsg('HHStocks: Deviations in FOC for v')
             dev2
         end

         % ** Check EE form **
         dev1 = zeros(1,N-1);
         for i = 1 : N-1
             dev1(i) = dHdx(i+1) / (1-sxx(i+1)) - (1-ddhx)*dHdx(i)...
                 /(1-sxx(i))/Rx(i+1);
         end
         if any(abs(dev1) > 1e-4)
             warnmsg('HHStocks: Deviations in EE for x')
             dev1
         %else
         %    maxDev = max(abs(dev1));
         %    disp(['EE satisfied. Maximum deviation: ', num2str(maxDev)]);
         end
     end


     if 0
         % ******  SHOW  ******

         disp('Age     -x-     -v-     -a-     -h-    -Rx-');
         for i = 1 : N
             tmp = sprintf('%3i %7.3f %7.3f %7.3f %7.3f %7.3f', i, ...
                 x(i), v(i), a(i), h(i), Rx(i));
             disp(tmp);
         end
         disp('    -costx-    -h1-')
         tmp = sprintf('    %7.3f %7.3f', costx, h1);
         disp(tmp);
     end
 end

% *** end function ***
